@@ -1,3 +1,5 @@
+import typing
+
 from mitmproxy import flowfilter
 from mitmproxy import exceptions
 from mitmproxy import ctx
@@ -7,6 +9,17 @@ class Intercept:
     def __init__(self):
         self.filt = None
 
+    def load(self, loader):
+        loader.add_option(
+            "intercept_active", bool, False,
+            "Intercept toggle"
+        )
+
+        loader.add_option(
+            "intercept", typing.Optional[str], None,
+            "Intercept filter expression."
+        )
+
     def configure(self, updated):
         if "intercept" in updated:
             if not ctx.options.intercept:
@@ -36,3 +49,7 @@ class Intercept:
 
     def response(self, f):
         self.process_flow(f)
+
+    def tcp_message(self, f):
+        if self.filt and ctx.options.intercept_active:
+            f.intercept()
