@@ -1,8 +1,6 @@
 # Default view cutoff *in lines*
 import typing
 
-VIEW_CUTOFF = 512
-
 KEY_MAX = 30
 
 TTextType = typing.Union[str, bytes]  # FIXME: This should be either bytes or str ultimately.
@@ -11,8 +9,8 @@ TViewResult = typing.Tuple[str, typing.Iterator[TViewLine]]
 
 
 class View:
-    name = None  # type: str
-    content_types = []  # type: typing.List[str]
+    name: typing.ClassVar[str]
+    content_types: typing.ClassVar[typing.List[str]] = []
 
     def __call__(self, data: bytes, **metadata) -> TViewResult:
         """
@@ -37,6 +35,37 @@ class View:
         raise NotImplementedError()  # pragma: no cover
 
 
+def format_pairs(
+        items: typing.Iterable[typing.Tuple[TTextType, TTextType]]
+) -> typing.Iterator[TViewLine]:
+
+    """
+    Helper function that accepts a list of (k,v) pairs into a list of
+    [
+        ("key", key    )
+        ("value", value)
+    ]
+    where key is padded to a uniform width
+    """
+
+    max_key_len = max((len(k[0]) for k in items), default=0)
+    max_key_len = min((max_key_len, KEY_MAX), default=0)
+
+    for key, value in items:
+        if isinstance(key, bytes):
+
+            key += b":"
+        else:
+            key += ":"
+
+        key = key.ljust(max_key_len + 2)
+
+        yield [
+            ("header", key),
+            ("text", value)
+        ]
+
+
 def format_dict(
         d: typing.Mapping[TTextType, TTextType]
 ) -> typing.Iterator[TViewLine]:
@@ -49,18 +78,7 @@ def format_dict(
     entries, where key is padded to a uniform width.
     """
 
-    max_key_len = max((len(k) for k in d.keys()), default=0)
-    max_key_len = min((max_key_len, KEY_MAX), default=0)
-    for key, value in d.items():
-        if isinstance(key, bytes):
-            key += b":"
-        else:
-            key += ":"
-        key = key.ljust(max_key_len + 2)
-        yield [
-            ("header", key),
-            ("text", value)
-        ]
+    return format_pairs(d.items())
 
 
 def format_text(text: TTextType) -> typing.Iterator[TViewLine]:
