@@ -135,8 +135,8 @@ def _read_set_cookie_pairs(s: str, off=0) -> Tuple[List[TPairs], int]:
         off: start offset
         specials: attributes that are treated specially
     """
-    cookies = []  # type: List[TPairs]
-    pairs = []  # type: TPairs
+    cookies: List[TPairs] = []
+    pairs: TPairs = []
 
     while True:
         lhs, off = _read_key(s, off, ";=,")
@@ -159,13 +159,17 @@ def _read_set_cookie_pairs(s: str, off=0) -> Tuple[List[TPairs], int]:
                 if len(rhs) <= 3:
                     trail, off = _read_value(s, off + 1, ";,")
                     rhs = rhs + "," + trail
-        if rhs or lhs:
+
+            # as long as there's a "=", we consider it a pair
             pairs.append([lhs, rhs])
 
-            # comma marks the beginning of a new cookie
-            if off < len(s) and s[off] == ",":
-                cookies.append(pairs)
-                pairs = []
+        elif lhs:
+            pairs.append([lhs, rhs])
+
+        # comma marks the beginning of a new cookie
+        if off < len(s) and s[off] == ",":
+            cookies.append(pairs)
+            pairs = []
 
         off += 1
 
@@ -300,7 +304,7 @@ def refresh_set_cookie_header(c: str, delta: int) -> str:
             e = email.utils.parsedate_tz(attrs["expires"])
             if e:
                 f = email.utils.mktime_tz(e) + delta
-                attrs.set_all("expires", [email.utils.formatdate(f)])
+                attrs.set_all("expires", [email.utils.formatdate(f, usegmt=True)])
             else:
                 # This can happen when the expires tag is invalid.
                 # reddit.com sends a an expires tag like this: "Thu, 31 Dec
