@@ -7,6 +7,8 @@ from mitmproxy.net import tls
 CONF_DIR = "~/.mitmproxy"
 CONF_BASENAME = "mitmproxy"
 LISTEN_PORT = 8080
+CONTENT_VIEW_LINES_CUTOFF = 512
+KEY_SIZE = 2048
 
 
 class Options(optmanager.OptManager):
@@ -67,6 +69,10 @@ class Options(optmanager.OptManager):
             regular expression and matched on the ip or the hostname.
             """
         )
+        self.add_option(
+            "allow_hosts", Sequence[str], [],
+            "Opposite of --ignore-hosts."
+        )
         self.add_option(
             "listen_host", str, "",
             "Address to bind proxy to."
@@ -162,5 +168,18 @@ class Options(optmanager.OptManager):
             communication contents are printed to the log in verbose mode.
             """
         )
+        self.add_option(
+            "content_view_lines_cutoff", int, CONTENT_VIEW_LINES_CUTOFF,
+            """
+            Flow content view lines limit. Limit is enabled by default to
+            speedup flows browsing.
+            """
+        )
+        self.add_option(
+            "key_size", int, KEY_SIZE,
+            """
+            TLS key size for certificates and CA.
+            """
+        )
 
         self.update(**kwargs)
