@@ -35,6 +35,7 @@ class DummyServer:
 class ProxyServer(tcp.TCPServer):
     allow_reuse_address = True
     bound = True
+    channel: controller.Channel
 
     def __init__(self, config: config.ProxyConfig) -> None:
         """
@@ -53,7 +54,6 @@ class ProxyServer(tcp.TCPServer):
             raise exceptions.ServerException(
                 'Error starting proxy server: ' + repr(e)
             ) from e
-        self.channel: controller.Channel = None
 
     def set_channel(self, channel):
         self.channel = channel
@@ -131,7 +131,7 @@ class ConnectionHandler:
                 self.log(repr(e), "debug")
             elif isinstance(e, exceptions.InvalidServerCertificate):
                 self.log(str(e), "warn")
-                self.log("Invalid certificate, closing connection. Pass --insecure to disable validation.", "warn")
+                self.log("Invalid certificate, closing connection. Pass --ssl-insecure to disable validation.", "warn")
             else:
                 self.log(str(e), "warn")
 
