@@ -1,7 +1,7 @@
-import webbrowser
-
 import tornado.httpserver
 import tornado.ioloop
+from tornado.platform.asyncio import AsyncIOMainLoop
+
 from mitmproxy import addons
 from mitmproxy import log
 from mitmproxy import master
@@ -102,60 +102,12 @@ class WebMaster(master.Master):
         )
 
     def run(self):  # pragma: no cover
-
+        AsyncIOMainLoop().install()
         iol = tornado.ioloop.IOLoop.instance()
-
         http_server = tornado.httpserver.HTTPServer(self.app)
-        http_server.listen(self.options.web_port, self.options.web_iface)
-
-        iol.add_callback(self.start)
-        tornado.ioloop.PeriodicCallback(lambda: self.tick(timeout=0), 5).start()
-
-        web_url = "http://{}:{}/".format(self.options.web_iface, self.options.web_port)
-        self.add_log(
-            "Web   server listening at {}".format(web_url),
-            "info"
+        http_server.listen(self.options.web_port, self.options.web_host)
+        web_url = "http://{}:{}/".format(self.options.web_host, self.options.web_port)
+        self.log.info(
+            "Web server listening at {}".format(web_url),
         )
-
-        if self.options.web_open_browser:
-            success = open_browser(web_url)
-            if not success:
-                self.add_log(
-                    "No web browser found. Please open a browser and point it to {}".format(web_url),
-                    "info"
-                )
-        try:
-            iol.start()
-        except KeyboardInterrupt:
-            self.shutdown()
-
-    def shutdown(self):
-        tornado.ioloop.IOLoop.instance().stop()
-        super().shutdown()
-
-
-def open_browser(url: str) -> bool:
-    """
-    Open a URL in a browser window.
-    In contrast to webbrowser.open, we limit the list of suitable browsers.
-    This gracefully degrades to a no-op on headless servers, where webbrowser.open
-    would otherwise open lynx.
-
-    Returns:
-        True, if a browser has been opened
-        False, if no suitable browser has been found.
-    """
-    browsers = (
-        "windows-default", "macosx",
-        "google-chrome", "chrome", "chromium", "chromium-browser",
-        "firefox", "opera", "safari",
-    )
-    for browser in browsers:
-        try:
-            b = webbrowser.get(browser)
-        except webbrowser.Error:
-            pass
-        else:
-            b.open(url)
-            return True
-    return False
+        self.run_loop(iol.start)
