@@ -81,6 +81,8 @@ def typespec_to_str(typespec: typing.Any) -> str:
         t = 'optional str'
     elif typespec == typing.Sequence[str]:
         t = 'sequence of str'
+    elif typespec == typing.Optional[int]:
+        t = 'optional int'
     else:
         raise NotImplementedError
     return t
