@@ -2,13 +2,13 @@ import os
 import subprocess
 import sys
 
-VERSION = "5.1.1"
+VERSION = "5.2"
 PATHOD = "pathod " + VERSION
 MITMPROXY = "mitmproxy " + VERSION
 
 # Serialization format version. This is displayed nowhere, it just needs to be incremented by one
 # for each change in the file format.
-FLOW_FORMAT_VERSION = 7
+FLOW_FORMAT_VERSION = 8
 
 
 def get_dev_version() -> str:
@@ -20,6 +20,14 @@ def get_dev_version() -> str:
 
     here = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
     try:
+        # Check that we're in the mitmproxy repository: https://github.com/mitmproxy/mitmproxy/issues/3987
+        # cb0e3287090786fad566feb67ac07b8ef361b2c3 is the first mitmproxy commit.
+        subprocess.run(
+            ['git', 'cat-file', '-e', 'cb0e3287090786fad566feb67ac07b8ef361b2c3'],
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL,
+            cwd=here,
+            check=True)
         git_describe = subprocess.check_output(
             ['git', 'describe', '--tags', '--long'],
             stderr=subprocess.STDOUT,
