@@ -1,8 +1,7 @@
+import os
 import string
 import random
 import mmap
-
-import six
 import sys
 
 DATATYPES = dict(
@@ -15,11 +14,11 @@ DATATYPES = dict(
     punctuation=string.punctuation.encode(),
     whitespace=string.whitespace.encode(),
     ascii=string.printable.encode(),
-    bytes=bytes(bytearray(range(256)))
+    bytes=bytes(range(256))
 )
 
 
-class TransformGenerator(object):
+class TransformGenerator:
 
     """
         Perform a byte-by-byte transform another generator - that is, for each
@@ -52,12 +51,10 @@ def rand_byte(chars):
     """
     # bytearray has consistent behaviour on both Python 2 and 3
     # while bytes does not
-    if six.PY2:
-        return random.choice(chars)
     return bytes([random.choice(chars)])
 
 
-class RandomGenerator(object):
+class RandomGenerator:
 
     def __init__(self, dtype, length):
         self.dtype = dtype
@@ -76,21 +73,21 @@ class RandomGenerator(object):
         return "%s random from %s" % (self.length, self.dtype)
 
 
-class FileGenerator(object):
-
+class FileGenerator:
     def __init__(self, path):
-        self.path = path
-        self.fp = open(path, "rb")
-        self.map = mmap.mmap(self.fp.fileno(), 0, access=mmap.ACCESS_READ)
+        self.path = os.path.expanduser(path)
 
     def __len__(self):
-        return len(self.map)
+        return os.path.getsize(self.path)
 
     def __getitem__(self, x):
-        if isinstance(x, slice):
-            return self.map.__getitem__(x)
-        # A slice of length 1 returns a byte object (not an integer)
-        return self.map.__getitem__(slice(x, x + 1 or self.map.size()))
+        with open(self.path, mode="rb") as f:
+            if isinstance(x, slice):
+                with mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as mapped:
+                    return mapped.__getitem__(x)
+            else:
+                f.seek(x)
+                return f.read(1)
 
     def __repr__(self):
         return "<%s" % self.path
